package Tietorakenteet;

import java.io.OutputStream;
import java.io.PrintStream;
import java.util.Random;

import fi.jyu.mit.ohj2.Mjonot;

/**
 * @author MHo
 * @version 20.2.2020
 * Jasen -luokka, joka lisää jäsenen rekisteriin.
 */
public class Hiihto {
    private int id;
    private int jasenID;
    private String pvm = "";
    private double matka;
    private String aika = "";
    private double lampotila;
    private double keskisyke;
    private double maksimisyke;
    private double kalorit;
    
    private static int juoksevaNro = 1;
    
    
    /**
     * Muodostaja
     * @param jasenID jolle hiihtosuoritus lisätään
     */
    public Hiihto(int jasenID){
        id = juoksevaNro;
        this.jasenID = jasenID;
        pvm = "5.3.2020";
        matka = new Random().nextInt((50 - 1) + 1) + 1; //arpoo matkan yksilöimistä varten
        aika = "1:30";
        lampotila = new Random().nextInt((5 - (-20)) + 1) + (-20); //arpoo lämpötilan yksilöimistä varten
        keskisyke = 120;
        maksimisyke = 180;
        kalorit = 100;
        juoksevaNro++;
    }
    
    
    /**
     * Selvittää harrastuksen tiedot | erotellusta merkkijonosta.
     * Pitää huolen että seuraavaNro on suurempi kuin tuleva tunnusnro.
     * @param rivi josta harrastuksen tiedot otetaan
     * @example
     * <pre name="test">
     *   Hiihto h = new Hiihto(0);
     *   h.parse("1  | 5 |  10.3.2020 | 20 | 1:30 | 150 ");
     *   h.getJasenID() === 5;
     *   h.toString().startsWith("1|5|10.3.2020|20|");
     *   int n = h.getJasenID();
     *   h.parse(""+(n+20));
     *   h.getJasenID() === n+20;
     *   h.toString().startsWith("+(n+20)");
     * </pre>
     */
    public void parse(String rivi) {
        StringBuffer sb = new StringBuffer(rivi);
        //setTunnusNro(Mjonot.erota(sb, '|', getTunnusNro()));
        id = Mjonot.erota(sb, '|', id);
        jasenID = Mjonot.erota(sb, '|', jasenID);
        pvm = Mjonot.erota(sb, '|', pvm);
        matka = Mjonot.erota(sb, '|', matka);
        aika = Mjonot.erota(sb, '|', aika);
        keskisyke = Mjonot.erota(sb, '|', keskisyke);
        maksimisyke = Mjonot.erota(sb, '|', maksimisyke);
        kalorit = Mjonot.erota(sb, '|', kalorit);
        juoksevaNro++;
    }
    
    
    /**
     * Palauttaa harrastuksen tiedot merkkijonona jonka voi tallentaa tiedostoon.
     * @return harrastus tolppaeroteltuna merkkijonona 
     * @example
     * <pre name="test">
     *   Hiihto h = new Hiihto(0);
     *   h.parse("   1   |  3  |   31.3.2020  | 10 | 1:30 ");
     *   h.toString()    === "1|3|31.3.2020|10|1:30";
     * </pre>
     */
    @Override
    public String toString() {
        return id + "|" + jasenID + "|" + pvm + "|" + matka + "|" + aika + "|" + lampotila + "|" + keskisyke + "|"
                + maksimisyke + "|" + kalorit;
    }
    
    
    /**
     * @return hiihdon jasen-ID
     */
    public int getJasenID() {
        return jasenID;
    }
    
    
    /**
     * Haetaan hiihdon kaikki tiedot
     * @return hiihtosuorituksen tiedot merkkijonotaulukkona
     * @example
     * <pre name="test">
     * #import java.util.Arrays;
     * Hiihto eka = new Hiihto(1);
     * Arrays.toString(eka.getTiedot()) === "[5.3.2020, 20, 1:30, -5, 120, 180, 100]";
     * </pre>
     */
    public String[] getTiedot() {
        String[] tiedot = new String[] {pvm, Double.toString(matka), aika, Double.toString(lampotila),
                Double.toString(keskisyke), Double.toString(maksimisyke), Double.toString(kalorit)};
        return tiedot;
    }
    
    
    /**
     * Tulostetaan hiihdon tiedot
     * @param out tietovirta johon tulostetaan
     */
    public void tulosta(PrintStream out) {
        out.println(String.format("%03d", jasenID, 3) + " " + pvm + " " + matka + "km aika: " + aika
                + " Lämpötila: " + lampotila + "  Keskisyke: " + keskisyke + " Maksimisyke: " + maksimisyke + " Kalorit: " + kalorit);
    }
    
    
    /**
     * Tulostetaan hiihdon tiedot
     * @param os tietovirta johon tulostetaan
     */
    public void tulosta(OutputStream os) {
        tulosta(new PrintStream(os));
    }

    
    /**
     * @param args ei käytössä
     */
    public static void main(String args[]) {
        Hiihto lenkki = new Hiihto(1);
        lenkki.tulosta(System.out);
    }
}