package Tietorakenteet;

import java.io.OutputStream;
import java.io.PrintStream;
import java.util.Random;

import fi.jyu.mit.ohj2.Mjonot;

/**
 * @author MHo
 * @version 20.2.2020
 * Jasen -luokka, joka lisää jäsenen rekisteriin.
 */
public class Jasen {
    private int id;
    private String nimi = "";
    private String syntAika = "";
    private String pKunta = "";
    private int puh;
    private String sPosti = "";
    private int liittymisV;
    
    private static int juoksevaNro = 1;
    
    
    /**
     * Muodostaja
     */
    public Jasen(){
        id = juoksevaNro;
        nimi = "Nokka Räkä";
        syntAika = "01.08.1960";
        pKunta = "Lahti";
        puh = 0401234567;
        sPosti = "nr@niilot.fi";
        liittymisV = new Random().nextInt((2020 - 1980) + 1) + 1980; //arpoo liittymisvuoden yksilöimistä varten
        juoksevaNro++;
    }
    
    
    /**
     * Muodostaja merkkijonosta
     * @param rivi tiedostosta
     */
    public void parseJasen(String rivi){
        StringBuilder sb = new StringBuilder(rivi);
        id = Mjonot.erota(sb, '|', juoksevaNro);
        nimi = Mjonot.erota(sb, '|', nimi);
        syntAika = Mjonot.erota(sb, '|', syntAika);
        pKunta = Mjonot.erota(sb, '|', pKunta);
        puh = Mjonot.erota(sb, '|', puh);
        sPosti = Mjonot.erota(sb, '|', sPosti);
        liittymisV = Mjonot.erota(sb, '|', liittymisV);
    }
    
    
    @Override
    /**
     * Palauttaa jäsenen tiedot merkkijonona jonka voi tallentaa tiedostoon.
     * @return jäsen tolppaeroteltuna merkkijonona 
     * @example
     * <pre name="test">
     *   Jasen jasen = new Jasen();
     *   jasen.parseJasen("   3  |  Ankka Aku   | 31.3.2020  | Jykyla | 040123 | aku@posti.fi");
     *   jasen.toString().startsWith("3|Ankka Aku|31.3.2020|Jykyla|") === true; // on enemmän kuin 3 kenttää, siksi loppu |
     * </pre>  
     */
    public String toString() {
        return "" +
                id + "|" +
                nimi + "|" +
                syntAika + "|" +
                pKunta + "|" +
                puh + "|" +
                sPosti + "|" +
                liittymisV;
    }
    
    
    /**
     * @return jäsenen ID-numero
     */
    public int getID() {
        return id;
    }
    
    
    /**
     * @return jäsenen nimi
     */
    public String getNimi() {
        return nimi;
    }

    
    /**
     * Haetaan henkilön tiedot
     * @return jasenen tiedot merkkijono -taulukkona
     * @example
     * <pre name="test">
     * #import java.util.Arrays;
     * Jasen testi = new Jasen();
     * Arrays.toString(testi.getTiedot()) === "[Nokka Räkä, 01.08.1960, Lahti, 0401234567, nr@niilot.fi, 2000]";
     * </pre>
     */
    public String[] getTiedot() {
        String[] tiedot = new String[] {nimi, syntAika, pKunta, Integer.toString(puh), sPosti, Integer.toString(liittymisV)};
        return tiedot;
    }
    
    
    /**
     * Tulostetaan henkilön tiedot
     * @param out tietovirta johon tulostetaan
     */
    public void tulosta(PrintStream out) {
        out.println(String.format("%03d", id, 3) + "  " + nimi);
        out.println(syntAika + "  " + pKunta + " " + puh + "  " + sPosti);
        out.print("Liittynyt " + liittymisV + ".");
    }
    
    
    /**
     * Tulostetaan henkilön tiedot
     * @param os tietovirta johon tulostetaan
     */
    public void tulosta(OutputStream os) {
        tulosta(new PrintStream(os));
    }

    
    /**
     * @param args ei käytössä
     */
    public static void main(String args[]) {
        Jasen nokka = new Jasen();
        nokka.tulosta(System.out);
    }
}