package Tietorakenteet;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;

/**
 * @author MHo
 * @version 20.2.2020
 * Hiihtorekisterin jasenet, joka lisää ja poistaa jäseniä.
 */
public class Jasenet {
    private static int MAX_JASENIA = 5;
    private int lkm = 0;
    private String tiedostonNimi = "nimet.dat";
    private Jasen alkiot[] = new Jasen[MAX_JASENIA];
    //private ArrayList<Jasen> jasenisto = new ArrayList<Jasen>();
    
    
    /**
     * Muodostaja
     */
    public Jasenet(){
       //
    }
    
    
    /**
     * Lisää uuden jäsenen tietorakenteeseen.  Ottaa jäsenen omistukseensa.
     * @param jasen lisätäävän jäsenen viite.  Huom tietorakenne muuttuu omistajaksi
     * @example
     * <pre name="test">
     * Jasenet jasenet = new Jasenet();
     * Jasen aku1 = new Jasen(), aku2 = new Jasen();
     * jasenet.getLkm() === 0;
     * jasenet.lisaa(aku1); jasenet.getLkm() === 1;
     * jasenet.lisaa(aku2); jasenet.getLkm() === 2;
     * jasenet.lisaa(aku1); jasenet.getLkm() === 3;
     * jasenet.anna(0) === aku1;
     * jasenet.anna(1) === aku2;
     * jasenet.anna(2) === aku1;
     * jasenet.anna(1) == aku1 === false;
     * jasenet.anna(1) == aku2 === true;
     * jasenet.anna(3) === aku1; #THROWS IndexOutOfBoundsException 
     * jasenet.lisaa(aku1); jasenet.getLkm() === 4;
     * jasenet.lisaa(aku1); jasenet.getLkm() === 5;
     * </pre>
     */
    public void lisaa(Jasen jasen){
        if (lkm >= alkiot.length) {
        Jasen uusi[] = new Jasen[MAX_JASENIA+10];
        MAX_JASENIA = uusi.length;
           for (int i = 0; i < alkiot.length; i++) {
           uusi[i] = alkiot[i];
           }
           alkiot = uusi;
        }
        alkiot[lkm] = jasen;
        lkm++;
    }
    
    
    /**
     * Palauttaa viitteen i:teen jäseneen.
     * @param i monennenko jäsenen viite halutaan
     * @return viite jäseneen, jonka indeksi on i
     * @throws IndexOutOfBoundsException jos i ei ole sallitulla alueella  
     * @example
     * <pre name="test">
     * Jasen testi = new Jasen();
     * Jasen toinen = new Jasen();
     * Jasenet jasenet = new Jasenet();
     * jasenet.lisaa(testi);
     * jasenet.lisaa(toinen);
     * jasenet.anna(0) === testi;
     * jasenet.anna(1) === toinen;
     * jasenet.anna(8) === toinen; #THROWS IndexOutOfBoundsException
     * </pre>
     */
    public Jasen anna(int i) throws IndexOutOfBoundsException {
        
        if (i < 0 || lkm <= i)
            throw new IndexOutOfBoundsException("Laiton indeksi: " + i);
        return alkiot[i];
    }
    
    
    /**
     * Lukee jäsenistön tiedostosta. 
     * @throws SailoException jos lukeminen epäonnistuu
     * @example
     * <pre name="test">
     * #THROWS SailoException 
     * #import java.io.File;
     * 
     *  Jasenet jasenet = new Jasenet();
     *  Jasen aku1 = new Jasen(), aku2 = new Jasen();
     *  File ftied = new File("nimet.dat");
     *  ftied.delete();
     *  jasenet.lueTiedostosta(); #THROWS SailoException
     *  jasenet.lisaa(aku1);
     *  jasenet.lisaa(aku2);
     *  jasenet.tallenna();
     *  jasenet = new Jasenet();            // Poistetaan vanhat luomalla uusi
     *  jasenet.lueTiedostosta();  // johon ladataan tiedot tiedostosta.
     *  jasenet.lisaa(aku2);
     *  jasenet.tallenna();
     *  ftied.delete() === true;
     *  File fbak = new File("nimet.bak");
     *  fbak.delete() === true;
     * </pre>
     */
    public void lueTiedostosta() throws SailoException {
        //setTiedostonPerusNimi(tied);
        try ( BufferedReader fi = new BufferedReader(new FileReader("nimet.dat"))) {
            tiedostonNimi = fi.readLine();
            if ( tiedostonNimi == null ) throw new SailoException("Kerhon nimi puuttuu");
            String rivi = fi.readLine();
            if ( rivi == null ) throw new SailoException("Maksimikoko puuttuu");
            // int maxKoko = Mjonot.erotaInt(rivi,10); // tehdään jotakin

            while ( (rivi = fi.readLine()) != null ) {
                rivi = rivi.trim();
                if ( "".equals(rivi) || rivi.charAt(0) == ';' ) continue;
                Jasen jasen = new Jasen();
                jasen.parseJasen(rivi); // voisi olla virhekäsittely
                lisaa(jasen);
            }
            //muutettu = false;
        } catch ( FileNotFoundException e ) {
            throw new SailoException("Tiedosto " + tiedostonNimi + " ei aukea");
        } catch ( IOException e ) {
            throw new SailoException("Ongelmia tiedoston kanssa: " + e.getMessage());
            }
    }


    /**
     * Tallentaa jäsenistön tiedostoon.  
     * Tiedoston muoto:
     * <pre>
     * Hiihtorekisteri
     * 5
     * 1|Henri Havu|13.3.2020|Haparanda|040546|havu@niilot.fi|1990
     * 2|Kalle Kone|12.5.2021|Torneå|040123|kone@niilot.fi|1989
     * </pre>
     * @throws SailoException jos talletus epäonnistuu
     */

    public void tallenna() throws SailoException {
        //if ( !muutettu ) return;

        File fbak = new File("nimet.bak");
        File ftied = new File("nimet.dat");
        fbak.delete(); // if .. System.err.println("Ei voi tuhota");
        ftied.renameTo(fbak); // if .. System.err.println("Ei voi nimetä");

        try ( PrintWriter fo = new PrintWriter(new FileWriter(ftied.getCanonicalPath())) ) {
            fo.println("nimet.dat");
            fo.println(MAX_JASENIA);
            //fo.println(alkiot.length);
            for (Jasen jasen : alkiot) {
                if (jasen == null) continue;
                fo.println(jasen.toString());
            }
            // catch ( IOException e ) { // ei heitä poikkeusta
            // throw new SailoException("Tallettamisessa ongelmia: " + e.getMessage());
        } catch ( FileNotFoundException ex ) {
            throw new SailoException("Tiedosto " + ftied.getName() + " ei aukea");
        } catch ( IOException ex ) {
            throw new SailoException("Tiedoston " + ftied.getName() + " kirjoittamisessa ongelmia");
        }
        //muutettu = false;
    }


    /**
     * Palauttaa kerhon jäsenten lukumäärän
     * @return jäsenten lukumäärä
     * @example
     * <pre name="test">
     * Jasen eka = new Jasen();
     * Jasen toka = new Jasen();
     * Jasenet jasenet = new Jasenet();
     * jasenet.lisaa(eka);
     * jasenet.lisaa(toka);
     * jasenet.getLkm() === 2;
     * </pre>
     */
    public int getLkm() {
        return lkm;
    }
    
    
    /**
     * @param args ei käytössä
     */
    public static void main(String args[]) {
        Jasenet jasenet = new Jasenet();

        Jasen nokka = new Jasen(), raka = new Jasen();

        try {
            jasenet.lisaa(nokka);
            jasenet.lisaa(raka);

            System.out.println("========== Jäsenet testi ==============");

            for (int i=0; i<jasenet.getLkm(); i++) {
                Jasen jasen = jasenet.anna(i);
                System.out.println("Jäsen nro: " + i);
                jasen.tulosta(System.out);
            }

        } catch ( Exception ex ) {
            System.out.println(ex.getMessage());
        }
    }
}