package Tietorakenteet;

import java.util.ArrayList;

/**
 * @author MHo
 * @version 20.2.2020
 * Hiihtorekisterin jasenet, joka lisää ja poistaa jäseniä.
 */
public class Rekisteri {
    private Jasenet jasenet = new Jasenet();
    private Hiihdot hiihdot = new Hiihdot();
    
    
    /**
     * Muodostaja
     */
    public Rekisteri(){
        //
    }
    
    
    /**
     * Lisää jäsenen taulukkoon.
     * @param jasen lisättävän jäsenen viite
     * @example
     * <pre name="test">
     * Rekisteri rekisteri = new Rekisteri();
     * Jasen eka = new Jasen();
     * Jasen toka = new Jasen();
     * Jasenet jasenet = new Jasenet();
     * rekisteri.lisaa(eka);
     * rekisteri.lisaa(toka);
     * jasenet.getLkm() === 2;
     * </pre>
     */
    public void lisaa(Jasen jasen){
        jasenet.lisaa(jasen);
    }
    
    
    /**
     * @param i monennenko jäsenen viite haetaan
     * @return viite jäseneen, jonka indeksi on i
     * @example
     * <pre name="test">
     * Rekisteri rekisteri = new Rekisteri();
     * Jasen eka = new Jasen();
     * Jasen toka = new Jasen();
     * Jasenet jasenet = new Jasenet();
     * jasenet.lisaa(eka);
     * jasenet.lisaa(toka);
     * rekisteri.annaJasen(0) === eka;
     * </pre>
     */
    public Jasen annaJasen(int i) {
        return jasenet.anna(i);
    }
    
    
    /**
     * Lisää hiihto listaan.
     * @param hiihto lisättävän hiihdon viite
     * @example
     * <pre name="test">
     * Rekisteri rekisteri = new Rekisteri();
     * Hiihto eka = new Hiihto(1);
     * Hiihto toka = new Hiihto(1);
     * rekisteri.lisaaHiihto(eka);
     * rekisteri.lisaaHiihto(toka);
     * rekisteri.getHiihdot() === 2;
     * </pre>
     */
    public void lisaaHiihto(Hiihto hiihto){
        hiihdot.lisaa(hiihto);
    }
    
    
    /**
     * @param i monennenko jäsenen viite haetaan
     * @return viite jäseneen, jonka indeksi on i
     * @example
     * <pre name="test">
     * Rekisteri rekisteri = new Rekisteri();
     * Hiihto eka = new Hiihto(1);
     * Hiihto toka = new Hiihto(1);
     * rekisteri.lisaaHiihto(eka);
     * rekisteri.lisaaHiihto(toka);
     * rekisteri.annaHiihto(0) === eka;
     * </pre>
     */
    public Hiihto annaHiihto(int i) {
        return hiihdot.anna(i);
    }
    
    
    /**
     * @param id hiihtajan ID
     * @return halutun jäsenen kaikki hiihdot
     * @example
     * <pre name="test">
     * Rekisteri rekisteri = new Rekisteri();
     * Hiihto eka = new Hiihto(1);
     * Hiihto toka = new Hiihto(1);
     * Hiihto kolmas = new Hiihto(2);
     * rekisteri.lisaaHiihto(eka);
     * rekisteri.lisaaHiihto(toka);
     * rekisteri.lisaaHiihto(kolmas);
     * rekisteri.getJasenenHiihdot(1) === "[eka, toka]";
     * rekisteri.getJasenenHiihdot(2) === "[kolmas]";
     * </pre>
    */
    public ArrayList<Hiihto> getJasenenHiihdot(int id) {
        return hiihdot.getJasenenHiihdot(id);
    }
    
    
    /**
     * Lukee rekisterin tiedot tiedostosta
     * @throws SailoException jos lukeminen epäonnistuu
     * @example
     * <pre name="test">
     * #THROWS SailoException 
     * #import java.io.*;
     * #import java.util.*;
     * Rekisteri r = new Rekisteri();
     *  Jasen j1 = new Jasen(); Jasen j2 = new Jasen();
     *  Hiihto h1 = new Hiihto(1);
     *  Hiihto h2 = new Hiihto(1);
     *  Hiihto h3 = new Hiihto(2);
     *  Hiihto h4 = new Hiihto(2);
     *  File fjas  = new File("nimet.dat");
     *  File fhiih = new File("hiihdot.dat");  
     *  fjas.delete();
     *  fhiih.delete();
     *  r.lueTiedostosta(); #THROWS SailoException
     *  r.lisaa(j1);
     *  r.lisaa(j2);
     *  r.lisaaHiihto(h1);
     *  r.lisaaHiihto(h2);
     *  r.lisaaHiihto(h3);
     *  r.lisaaHiihto(h4);
     *  r.tallenna();
     *  r = new Rekisteri();
     *  r.lueTiedostosta();
     *  ArrayList<Hiihto> j1h = r.getJasenenHiihdot(1);
     *  j1h.size() === 2;
     *  r.lisaa(j2);
     *  r.lisaaHiihto(h4);
     *  r.tallenna();
     *  fjas.delete()  === true;
     *  fhiih.delete() === true;
     *  File fjasbak = new File("nimet.bak");
     *  File fhbak = new File("hiihdot.bak");
     *  fjasbak.delete() === true;
     *  fhbak.delete() === true;
     * </pre>
     */

    public void lueTiedostosta() throws SailoException {
        jasenet = new Jasenet();
        hiihdot = new Hiihdot();
        jasenet.lueTiedostosta();
        hiihdot.lueTiedostosta();
    }

        
    /**
     * Tallettaa kerhon tiedot tiedostoon
     * Tallenttaa kerhon tiedot tiedostoon.  
     * Vaikka jäsenten tallettamien epäonistuisi, niin yritetään silti tallettaa
     * harrastuksia ennen poikkeuksen heittämistä.
     * @throws SailoException jos tallettamisessa ongelmia
     */
    public void tallenna() throws SailoException {
        String virhe = "";
        try {
            jasenet.tallenna();
        } catch ( SailoException ex ) {
            virhe = ex.getMessage();
        }

        try {
            hiihdot.tallenna();
        } catch ( SailoException ex ) {
            virhe += ex.getMessage();
        }
        if ( !"".equals(virhe) ) throw new SailoException(virhe);
    }

    
    /**
     * @return jäsenistön lkm
     * @example
     * <pre name="test">
     * Rekisteri rekisteri = new Rekisteri();
     * Jasen eka = new Jasen();
     * Jasen toka = new Jasen();
     * rekisteri.lisaa(eka);
     * rekisteri.lisaa(toka);
     * </pre>
     */
    public int getLkm() {
        return jasenet.getLkm();
    }
    
    
    /**
     * @return rekisterin hiihtosuoritusten lukumäärä
     * @example
     * <pre name="test">
     * Rekisteri rekisteri = new Rekisteri();
     * Hiihto eka = new Hiihto(1);
     * Hiihto toka = new Hiihto(1);
     * Hiihto kolmas = new Hiihto(2);
     * rekisteri.lisaaHiihto(eka);
     * rekisteri.lisaaHiihto(toka);
     * rekisteri.lisaaHiihto(kolmas);
     * rekisteri.getHiihdot() === 3;
     * </pre>
     */
    public int getHiihdot() {
        return hiihdot.getHiihdot();
    }
    /**
     * @param args ei käytössä
     */
    public static void main(String args[]) {
        Rekisteri rekisteri = new Rekisteri();
        
        try {
            // kerho.lueTiedostosta("kelmit");

            Jasen nokkaraka = new Jasen(), mietaa = new Jasen();
            

            rekisteri.lisaa(nokkaraka);
            rekisteri.lisaa(mietaa);

            System.out.println("============= Kerhon testi =================");

            for (int i = 0; i < rekisteri.getLkm(); i++) {
                Jasen jasen = rekisteri.annaJasen(i);
                System.out.println("Jäsen paikassa: " + i);
                jasen.tulosta(System.out);
                System.out.println();
                System.out.println();
            }

        } catch (Exception ex) {
            System.out.println(ex.getMessage());
        }
    }
}